Attribute VB_Name = "modGLSetup"
' modGLSetup: Routines for initializing OpenGL.  The GLSetup routine
' does my usual setup, which is a 640x480 viewport with a 4/3 frustum
' (just barely larger in the Y dimension than GLQuake) with a long
' view range.
'
' Function list:
'   GLSetup:        General initialization of OpenGL for the Form_Load event.
'   ChangeTo3D:     Changes to a 3D viewport and projection matrix.
'   ChangeTo2D:     Changes to a 2D viewport and projection matrix.
'   BeginOverlay:   Saves parameters and sets up for drawing overlays
'                   (mouse pointers, control panels, scores, etc.)
'   EndOverlay:     Restores settings saved by BeginOverlay.

Option Explicit

Public Sub GLSetup(ByVal blnEnableBackfaceCulling As Boolean, ByVal blnEnableBlending As Boolean, ByVal blnEnableFog As Boolean, ByVal blnEnableDepthTest As Boolean, ByVal blnEnableNormalize As Boolean, ByVal blnSmoothShading As Boolean)
    Dim fltFogColor(3) As GLfloat
    
    ChangeTo3D
    glCullFace GL_BACK
    glFrontFace GL_CCW
    If blnEnableBackfaceCulling = True Then glEnable GL_CULL_FACE Else glDisable GL_CULL_FACE
    If blnEnableFog = False Then glDisable GL_FOG Else glEnable GL_FOG
    glFogi GL_FOG_MODE, GL_EXP2
    glFogf GL_FOG_START, 200
    glFogf GL_FOG_END, 4000
    glFogf GL_FOG_DENSITY, 0.001
    glFogfv GL_FOG_COLOR, fltFogColor(0)
    glBlendFunc GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA
    If blnEnableBlending = False Then glDisable GL_BLEND Else glEnable GL_BLEND
    glClearColor 0, 0, 0, 0
    glDepthFunc GL_LEQUAL
    glClearDepth 1
    If blnEnableDepthTest = False Then glDisable GL_DEPTH_TEST Else glEnable GL_DEPTH_TEST
    If blnSmoothShading = False Then glShadeModel GL_SMOOTH Else glShadeModel GL_FLAT
    If blnEnableNormalize = False Then glDisable GL_NORMALIZE Else glEnable GL_NORMALIZE
    glPolygonMode faceFrontAndBack, pgmFILL
    glPixelStorei GL_UNPACK_ALIGNMENT, 1
End Sub
Public Sub ChangeTo3D()
    glViewport 0, 0, frmDisplay.ScaleWidth, frmDisplay.ScaleHeight
    glMatrixMode GL_PROJECTION
    glLoadIdentity
    If frmDisplay.ScaleHeight Mod frmDisplay.ScaleWidth <> 0 Then
        glFrustum -(frmDisplay.ScaleWidth / (frmDisplay.ScaleHeight Mod frmDisplay.ScaleWidth)), (frmDisplay.ScaleWidth / (frmDisplay.ScaleHeight Mod frmDisplay.ScaleWidth)), -(frmDisplay.ScaleHeight / (frmDisplay.ScaleHeight Mod frmDisplay.ScaleWidth)), (frmDisplay.ScaleHeight / (frmDisplay.ScaleHeight Mod frmDisplay.ScaleWidth)), 4, 4096
    Else
        glFrustum -4, 4, -4, 4, 4, 4096
    End If
    glMatrixMode mmModelView
    glLoadIdentity
End Sub
Public Sub ChangeTo2D(ByVal fltLeft As GLfloat, ByVal fltRight As GLfloat, ByVal fltBottom As GLfloat, ByVal fltTop As GLfloat, ByVal fltNear As GLfloat, ByVal fltFar As GLfloat)
    glMatrixMode GL_PROJECTION
    glLoadIdentity
    glOrtho fltLeft, fltRight, fltBottom, fltTop, fltNear, fltFar
    glMatrixMode GL_MODELVIEW
    glLoadIdentity
End Sub
Public Sub BeginOverlay()
    glPushAttrib GL_ENABLE_BIT Or GL_VIEWPORT_BIT Or GL_TRANSFORM_BIT
        glDisable GL_LIGHTING
        glDisable GL_FOG
        glDisable GL_DEPTH_TEST
        glDisable GL_BLEND
        glMatrixMode GL_PROJECTION
        glPushMatrix
            glLoadIdentity
            gluOrtho2D 0, frmDisplay.ScaleWidth, 0, frmDisplay.ScaleHeight
            glMatrixMode GL_MODELVIEW
            glPushMatrix
                glLoadIdentity
End Sub
Public Sub EndOverlay()
            glPopMatrix
            glMatrixMode GL_PROJECTION
        glPopMatrix
    glPopAttrib
End Sub
